function resultat = evaluation(model,file,th)
%
%	evaluation: Evaluate the quality of a model using a set of test connections
%
%	Description
%	EVAL = evaluation(MODEL,FILE,TH) takes as input a model (obtained from modelBehaviors),
%		labels the connections in FILE based on the sizes of the first model.pkts packets
%		and evaluate the quality of the model using the real application labels in FILE
%		TH specifies a threshold for cluster assignment (0<=TH<=1; TH=0: no thresholds)
%
%	Each connection is described in one line with the following syntax:
%	app	dport	pkt1	pkt2	...	pktN
%
%	The output is a structure EVAL with fields:
%		Mapping: The cluster to which each connection was assigned (0: unknown)
%
%		DominantOverall: Overall accuracy on the tested file for the Dominant heuristic
%		DominantClass: Application for each connection (numbers correspond to model.Applis,
%								0 to connections labeled 'unknown')
%		DominantMisclassif: Classification matrix
%			Each cell (i,j) reads: proportion of connection from app i classified as j
%			Last column: Proportion of False Positives
%
%		Dominant_OtherAppsMisclassed: Proportion of traffic that does not belong to any known app 
%						that has been misclassifed (among traffic not labeled 'unknown')
%						Always 1 if using a threshold and Unknown<1
%		
%		ClusterPortOverall: Overall accuracy for the Cluster+Port heuristic
%		ClusterPortClass: Application for each connection. In addition:
%			Class=0:  Unknown
%			Class=-1: Masquerade : Non std app on other std port		
%			Class=-2: Masquerade : Std App on non std port
%		ClusterPortMisclassif: Classification matrix
%			For N applications in model
%			Each cell (i,j) reads: proportion of connection from app i classified as j, for j<=N
%			Column N+1: Masquerade : Non std app on other std port
%			Column N+2: Masquerade : Std App on non std port
%			Column N+3 (last column): Proportion of false positives for each applications
%
%		ClusterPort_OtherAppsMisclassed: Proportion of traffic that does not belong to any known app 
%						 that has been misclassified (among traffic not labeled 'unknown')
%
%		ClusterPort_OtherAppsMasquerade: Proportion of traffic that does not belong to any known app 
%						 that has been labeled Masquerade (among traffic not labeled 'unknown')
%							Misclassified+Masquerade=1, always
%
%		Unknown: Proportion of connections labeled 'unknown'
%		Test: Tested connections

	addpath('netlab');

	pkts=model.pkts;
	switch (pkts)
		case 1
			[path dport p1]=textread(file,'%s\t%d\t%d%*[^\n]');
			seq=[p1];
		case 2
			[path dport p1 p2]=textread(file,'%s\t%d\t%d\t%d%*[^\n]');
			seq=[p1 p2];
		case 3
			[path dport p1 p2 p3]=textread(file,'%s\t%d\t%d\t%d\t%d%*[^\n]');
			seq=[p1 p2 p3];
		case 4
			[path dport p1 p2 p3 p4]=textread(file,'%s\t%d\t%d\t%d\t%d\t%d%*[^\n]');
			seq=[p1 p2 p3 p4];
		case 5
			[path dport p1 p2 p3 p4 p5]=textread(file,'%s\t%d\t%d\t%d\t%d\t%d\t%d%*[^\n]');
			seq=[p1 p2 p3 p4 p5];
	end

	
	pathNum=appliTonumber(model.Applis,path);

%	[probas mapping]=max(gmmlpost(model.gmm,seq),[],2);	
%	newprobas=probas.*(probas>=th);
%	knownIdx=find(newprobas>0);
%	unknownIdx=find(newprobas==0);

	[probas mapping]=max(gmmlactiv(model.gmm,seq),[],2);	
	knownIdx=find(probas>=th);
	unknownIdx=find(probas<=th);

	mapping(unknownIdx)=0;

% Dominant Analysis
	Class(knownIdx)=model.labelDom(mapping(knownIdx));
	Class(unknownIdx)=0;
	Class=Class';
	if (length(knownIdx)>0)
		napps=length(model.Applis);
		mat=zeros(napps,napps+1);
		for a=1:napps
			idx=find(pathNum(knownIdx)==a);
			idxnot=find(pathNum(knownIdx)~=a);
			for b=1:napps
				if (length(idx)==0)
					mat(a,b)=0;
				else
					mat(a,b)=sum(Class(knownIdx(idx))==b)/length(idx);
				end
			end
			mat(a,napps+1)=sum(Class(knownIdx(idxnot))==a)/length(knownIdx);
		end
		%Overall among connections assigned to a cluster
		resultat.DominantOverall=sum(Class(knownIdx)==pathNum(knownIdx))/length(knownIdx);
		resultat.DominantMisclassif=mat;

		%Other traffic
		newapps=find(pathNum(knownIdx)==0);
		resultat.Dominant_OtherAppsMisclassed=sum(Class(knownIdx(newapps))>0)/length(knownIdx);
	else
		resultat.DominantOverall=0;
	end
	resultat.DominantClass=Class;


% Port analysis
	idxPort=zeros(length(knownIdx),1);
	for i=1:length(knownIdx)
		idx=find(model.knownports==dport(knownIdx(i)));
		if (length(idx)>0)
			idxPort(i)=idx;
		end
	end
		
	Class=zeros(length(pathNum),1);

	if (length(knownIdx)>0)
		idxPortOK=find(idxPort>0);
		idxPortKO=find(idxPort==0);

		for f=1:length(idxPortOK)
			Class(knownIdx(idxPortOK(f)))=model.appFromPort(mapping(knownIdx(idxPortOK(f))),idxPort(idxPortOK(f)));
		end
		Class(knownIdx(idxPortKO))=model.DominantUnStd(mapping(knownIdx(idxPortKO)));

		resTmp=sum(Class(knownIdx)==pathNum(knownIdx))/length(knownIdx);
		napps=length(model.Applis);
		mat=zeros(napps,napps+3);
		for a=1:napps
			idx=find(pathNum(knownIdx)==a);
			idxnot=find(pathNum(knownIdx)~=a);
			for b=1:napps
				if (length(idx)==0)
					mat(a,b)=0;
				else
					mat(a,b)=sum(Class(knownIdx(idx))==b)/length(idx);
				end
			end
				
			if (length(idx)==0)
				mat(a,napps+1)=0;
				mat(a,napps+2)=0;
			else
				mat(a,napps+1)=sum(Class(knownIdx(idx))==-1)/length(idx);
				mat(a,napps+2)=sum(Class(knownIdx(idx))==-2)/length(idx);
			end
			mat(a,napps+3)=sum(Class(knownIdx(idxnot))==a)/length(knownIdx);
		end

	% col napp+1 : Masquerade : Non std app on other std port
	% col napp+2 : Masquerade : Std App on non std port
	% col napp+3 : False Positives

		%Other traffic
		newapps=find(pathNum(knownIdx)==0);
		resultat.ClusterPort_OtherAppsMisclassed=sum(Class(knownIdx(newapps))>0)/length(knownIdx);
		resultat.ClusterPort_OtherAppsMasquerade=sum(Class(knownIdx(newapps))<0)/length(knownIdx);


		resultat.ClusterPortOverall=resTmp;
		resultat.ClusterPortMisclassif=mat;
	else
		resultat.ClusterPortOverall=0;
	end

	resultat.ClusterPortClass=Class;
	resultat.Test.path=pathNum;
	resultat.Test.dport=dport;
	resultat.Test.seq=seq;

	resultat.Mapping=mapping;
	resultat.Unknown=1-length(knownIdx)/length(pathNum);
end
