function model=modelBehaviors(file,pkts,clusters)
%	modelBehaviors	Creates a model for the application behaviors found in a training set.
%
%	Description
%	MODEL=modelBehaviors(FILE,PKTS,CLUSTERS) takes as input a file (FILE)  containing a series of connections 
%	and the number of packets and clusters to consider in the model (PKT, CLUSTERS)
%
%	Each connection is described in one line with the following syntax:
%	appname	dport	pkt1	pkt2	...	pktN
%	
%	appname is the name of the application that generated a connection
%	dport is the server port used in the TCP connection
%	pkt1 ... pktN are the payload sizes of the TCP packets which have a payload.
%
%	For better results a similar number of connections should be used for each application
%
%	The output MODEL is a description of the behaviors found in the training set
%

	addpath('netlab');
	MINFLOWS=3;

	switch (pkts)
		case 1
			[path dport p1]=textread(file,'%s\t%d\t%d%*[^\n]');
			seq=[p1];
		case 2
			[path dport p1 p2 ]=textread(file,'%s\t%d\t%d\t%d%*[^\n]');
			seq=[p1 p2];
		case 3
			[path dport p1 p2 p3]=textread(file,'%s\t%d\t%d\t%d\t%d%*[^\n]');
			seq=[p1 p2 p3];
		case 4
			[path dport p1 p2 p3 p4]=textread(file,'%s\t%d\t%d\t%d\t%d\t%d%*[^\n]');
			seq=[p1 p2 p3 p4];
		case 5
			[path dport p1 p2 p3 p4 p5]=textread(file,'%s\t%d\t%d\t%d\t%d\t%d\t%d%*[^\n]');
			seq=[p1 p2 p3 p4 p5];
	end


	Applis=dedoubloneListe(path);

	pathNum=appliTonumber(Applis,path);

	clear mix gmmparams myclasses;
		
	opt=zeros(14,1);
	opt(1)=-1;
	opt(5)=1;
	opt(14)=100;
			
	gmmI=gmm(pkts,clusters,'diag');
	gmmK=gmminit(gmmI,seq,opt);
	[gmmparams void1 void2]=gmmlem(gmmK,seq,opt);
	[void myclasses]=max(gmmlpost(gmmparams,seq),[],2);
	clear void void1 void2;
	model.myclasses=myclasses;


	% Filtering : remove clusters with less than MINFLOWS for an app
	nApps=length(Applis);


	nbFlows=zeros(nApps,clusters);
	for c=1:clusters
		for a=1:nApps
			idxCluster=find(model.myclasses==c);
			nbFlows(a,c)=sum(pathNum(idxCluster)==a);
		end
	end

	idxOK=find(nbFlows>=MINFLOWS);

	classedFlows=[];
	nbFlowsFiltered=zeros(nApps,clusters);
	for i=1:length(idxOK)
		app=mod(idxOK(i)-1,nApps)+1;
		clust=floor((idxOK(i)-1)/nApps)+1;
		idxOKFlows=find((pathNum==app).*(model.myclasses==clust));
		classedFlows=[classedFlows ; idxOKFlows];
		nbFlowsFiltered(app,clust)=length(idxOKFlows);
	end
			
	cleanClustersIndex=find(sum(nbFlowsFiltered)>0);
			
	newClassif=model.myclasses;
	empty=find(sum(nbFlowsFiltered)==0);
	for i=1:length(empty)
		idx=find(model.myclasses>empty(i));
		newClassif(idx)=newClassif(idx)-1;
	end
		
	nbFlowsFiltered=nbFlowsFiltered(:,cleanClustersIndex);

	model.proportion=nbFlowsFiltered./repmat(sum(nbFlowsFiltered),nApps,1);
	model.clusterShare=nbFlowsFiltered./repmat(sum(nbFlowsFiltered')',1,length(cleanClustersIndex));
	model.nbFlows=nbFlowsFiltered;
			
	model.Size = sum(model.nbFlows);
	
	model.gmm=gmmparams;
	model.gmm.ncentres = length(cleanClustersIndex);
	model.gmm.priors = gmmparams.priors(:,cleanClustersIndex);
	model.gmm.centres = gmmparams.centres(cleanClustersIndex,:);
	model.gmm.covars = gmmparams.covars(cleanClustersIndex,:);
	model.nmi=nmi(newClassif(classedFlows),pathNum(classedFlows));

	idx=find(model.gmm.covars<1);
	model.gmm.covars(idx)=1;

	model.class=newClassif(classedFlows);

	[void,id]=sort(-model.proportion);
	model.labelDom=id(1,:)';

	model.Applis=Applis;
	model.pkts=pkts;
	model.clusters=length(model.Size);

	params;
	matN=zeros(length(Applis),1);
	for i=1:length(Applis)
		matN(i)=STDPORT(find(strcmp(APPLICATIONS,Applis(i))));
	end

	unstdApps=find(matN==0);
	newprop=model.proportion(unstdApps,:);

	[maxprop id]=sort(-newprop,1);
	okunstd=find(maxprop(1,:)<0);

	%Class=-2: Connection on non std ports  in this cluster correspond to masquerade traffic (only std apps)
	model.DominantUnStd=repmat(-2,model.clusters,1);
	model.DominantUnStd(okunstd)=unstdApps(id(1,okunstd))';
	
	[model.knownports order]=sort(PORTS);

	model.appFromPort=repmat(-1,model.clusters,length(PORTS));	
	tokeep=[];
	for j=1:length(PORTS)
		apps=appliTonumber(model.Applis,APPFROMPORT{order(j)});
		if (sum(apps)~=0)
			%Apps in model for this port
			tokeep=[tokeep j];
			okapps=find(apps>0);
			for i=1:model.clusters
				pp=model.proportion(apps(okapps),i);
				if (sum(pp)>0)
					[void idxapp]=max(pp);
					model.appFromPort(i,j)=apps(okapps(idxapp));
				end
			end
		end
	end
	model.knownports=model.knownports(tokeep);
	model.appFromPort=model.appFromPort(:,tokeep);


%	For SSL
	[model.SSL_knownports order]=sort(SSL_PORTS);

	model.SSL_appFromPort=repmat(-1,model.clusters,length(SSL_PORTS));	
	tokeep=[];
	for j=1:length(SSL_PORTS)
		apps=appliTonumber(model.Applis,SSL_APPFROMPORT{order(j)});
		if (sum(apps)~=0)
			%Apps in model for this port
			tokeep=[tokeep j];
			okapps=find(apps>0);
			for i=1:model.clusters
				pp=model.proportion(apps(okapps),i);
				if (sum(pp)>0)
					[void idxapp]=max(pp);
					model.SSL_appFromPort(i,j)=apps(okapps(idxapp));
				end
			end
		end
	end
	model.SSL_knownports=model.SSL_knownports(tokeep);
	model.SSL_appFromPort=model.SSL_appFromPort(:,tokeep);

	model.train.pathNum=pathNum;
	model.train.seq=seq;
end
